<?php

namespace Bydedi\Http;

use Bydedi\Http\Event\FilterResponseEvent;
use Bydedi\Http\Event\MatchRequestEvent;
use Bydedi\Http\Event\PostResponseEvent;
use Bydedi\Http\Exception\MethodNotAllowedHttpException;
use Bydedi\Http\Exception\NotFoundHttpException;
use Symfony\Component\DependencyInjection\ContainerAware;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Class Application
 * @package Bydedi\Http
 */
class Application extends ContainerAware
{

    /**
     * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
     */
    protected $dispatcher;
    /**
     * @var \Symfony\Component\HttpFoundation\Request
     */
    protected $request;

    /**
     * @param EventDispatcherInterface $dispatcher
     */
    public function setEventDispatcher(EventDispatcherInterface $dispatcher)
    {
        $this->dispatcher = $dispatcher;
    }

    /**
     * @param Request $request
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function handle(Request $request)
    {
        $this->request = $request;

        // try to match request with something
        $event = new MatchRequestEvent($request);

        $this->dispatcher->dispatch(HttpEvents::REQUEST, $event);

        // if the request find a repsonse like redirection(need auth), we send the response
        if ($event->hasResponse()) {
            return $event->getResponse();
        }

        $controller = $request->attributes->get('_controller');

        $controllerObject = new $controller($request, $this->container);

        try {
            $response = call_user_func_array([$controllerObject, $request->attributes->get('_action')], []);
        } catch (NotFoundHttpException $e) {

        } catch (MethodNotAllowedHttpException $e) {

        }

        $event = new FilterResponseEvent($request, $response);
        $this->dispatcher->dispatch(HttpEvents::RESPONSE, $event);

        return $event->getResponse();
    }

    /**
     * Send event terminate with request and response
     *
     * @param Request  $request  the current request
     * @param Response $response the reponse
     *
     * @return void
     */
    public function terminate(Request $request, Response $response)
    {
        $this->dispatcher->dispatch(HttpEvents::TERMINATE, new PostResponseEvent($request, $response));
    }
}
